@@ -122,48 +122,62 @@ namespace zypp
       }
 
       void
-      setQueryStringMap(const zypp::url::ParamMap &pmap) override
+      setQueryStringMap(const zypp::url::ParamMap &pmap, EEncoding eflag ) override
       {
         static const char * const keys[] = {
           "attrs", "scope", "filter", "exts", NULL
         };
 
-        // remove psep ("?") from safe chars
-        std::string join_safe;
-        std::string safe(config("safe_querystr"));
-        std::string psep(config("psep_querystr"));
-        for(std::string::size_type i=0; i<safe.size(); i++)
+        if ( eflag == url::E_DECODED )
         {
-          if( psep.find(safe[i]) == std::string::npos)
-            join_safe.append(1, safe[i]);
-        }
-
-        zypp::url::ParamVec pvec(4);
-        zypp::url::ParamMap::const_iterator p;
-        for(p=pmap.begin(); p!=pmap.end(); ++p)
-        {
-          bool found=false;
-          for(size_t i=0; i<4; i++)
+          // remove psep ("?") from safe chars
+          std::string join_safe;
+          std::string safe(config("safe_querystr"));
+          std::string psep(config("psep_querystr"));
+          for(std::string::size_type i=0; i<safe.size(); i++)
           {
-            if(p->first == keys[i])
+            if( psep.find(safe[i]) == std::string::npos)
+              join_safe.append(1, safe[i]);
+          }
+
+          zypp::url::ParamVec pvec(4);
+          zypp::url::ParamMap::const_iterator p;
+          for(p=pmap.begin(); p!=pmap.end(); ++p)
+          {
+            bool found=false;
+            for(size_t i=0; i<4; i++)
             {
-              found=true;
-              pvec[i] = zypp::url::encode(p->second, join_safe);
+              if(p->first == keys[i])
+              {
+                found=true;
+                pvec[i] = zypp::url::encode(p->second, join_safe);
+              }
+            }
+            if( !found)
+            {
+              ZYPP_THROW(url::UrlNotSupportedException(
+                str::form(_("Invalid LDAP URL query parameter '%s'"),
+                          p->first.c_str())
+              ));
             }
           }
-          if( !found)
-          {
-            ZYPP_THROW(url::UrlNotSupportedException(
-              str::form(_("Invalid LDAP URL query parameter '%s'"),
-                          p->first.c_str())
-            ));
-          }
+          setQueryStringVec(pvec);
+        }
+        else
+        {
+          setQueryString(
+            zypp::url::join(
+              pmap,
+              config("psep_querystr"),
+              config("vsep_querystr"),
+              config("safe_querystr"),
+              url::E_ENCODED
+            )
+          );
         }
-        setQueryStringVec(pvec);
       }
     };
 
-
     // ---------------------------------------------------------------
     // FIXME: hmm..
     class UrlByScheme
@@ -390,7 +404,7 @@ namespace zypp
     {
       std::string scheme = out[1];
       if (scheme.size() > 1)
-        scheme = scheme.substr(0, scheme.size()-1);
+        scheme.pop_back();
       std::string authority = out[2];
       if (authority.size() >= 2)
         authority = authority.substr(2);
@@ -834,7 +848,7 @@ namespace zypp
   void
   Url::setQueryStringMap(const zypp::url::ParamMap &pmap)
   {
-    m_impl->setQueryStringMap(pmap);
+    m_impl->setQueryStringMap(pmap, url::E_DECODED);
   }
 
   // -----------------------------------------------------------------
@@ -851,6 +865,12 @@ namespace zypp
     m_impl->delQueryParam(param);
   }
 
+  void
+  Url::delQueryParams(const std::set<std::string> &params)
+  {
+    m_impl->delQueryParams(params);
+  }
+
   // -----------------------------------------------------------------
   ViewOptions
   Url::getViewOptions() const
