@@ -52,6 +52,8 @@ extern "C"
 #include <zypp/solver/detail/SystemCheck.h>
 #include <zypp/solver/detail/SolutionAction.h>
 #include <zypp/solver/detail/SolverQueueItem.h>
+
+#include <utility>
 using std::endl;
 
 #define XDEBUG(x) do { if (base::logger::isExcessive()) XXX << x << std::endl;} while (0)
@@ -272,7 +274,7 @@ SATResolver::dumpOn( std::ostream & os ) const
 #undef OUTS
         os << "  focus	= "	<< _focus << endl;
         os << "  distupgrade	= "	<< _distupgrade << endl;
-        os << "  distupgrade_removeunsupported	= " << _distupgrade_removeunsupported << endl;
+        os << "  removeOrphaned	= " << _removeOrphaned << endl;
         os << "  solveSrcPackages	= "	<< _solveSrcPackages << endl;
         os << "  cleandepsOnRemove	= "	<< _cleandepsOnRemove << endl;
         os << "  fixsystem		= "	<< _fixsystem << endl;
@@ -285,8 +287,8 @@ SATResolver::dumpOn( std::ostream & os ) const
 //---------------------------------------------------------------------------
 
 // NOTE: flag defaults must be in sync with ZVARDEFAULT in Resolver.cc
-SATResolver::SATResolver (const ResPool & pool, sat::detail::CPool *satPool)
-    : _pool(pool)
+SATResolver::SATResolver (ResPool  pool, sat::detail::CPool *satPool)
+    : _pool(std::move(pool))
     , _satPool(satPool)
     , _satSolver(NULL)
     , _focus			( ZConfig::instance().solver_focus() )
@@ -302,7 +304,8 @@ SATResolver::SATResolver (const ResPool & pool, sat::detail::CPool *satPool)
     , _onlyRequires		(ZConfig::instance().solver_onlyRequires())
     , _ignorealreadyrecommended(true)
     , _distupgrade(false)
-    , _distupgrade_removeunsupported(false)
+    , _removeOrphaned(false)
+    , _removeUnneeded(false)
     , _dup_allowdowngrade	( ZConfig::instance().solver_dupAllowDowngrade() )
     , _dup_allownamechange	( ZConfig::instance().solver_dupAllowNameChange() )
     , _dup_allowarchchange	( ZConfig::instance().solver_dupAllowArchChange() )
@@ -332,13 +335,13 @@ SATResolver::pool (void) const
 // if data != NULL, set as APPL_LOW (from establishPool())
 
 static void
-SATSolutionToPool (PoolItem item, const ResStatus & status, const ResStatus::TransactByValue causer)
+SATSolutionToPool (const PoolItem& item, const ResStatus & status, const ResStatus::TransactByValue causer)
 {
     // resetting
     item.status().resetTransact (causer);
     item.status().resetWeak ();
 
-    bool r;
+    bool r = false;
 
     // installation/deletion
     if (status.isToBeInstalled()) {
@@ -634,10 +637,23 @@ void SATResolver::solverInitSetModeJobsAndFlags()
     if (_distupgrade) {
         queue_push( &(_jobQueue), SOLVER_DISTUPGRADE|SOLVER_SOLVABLE_ALL);
         queue_push( &(_jobQueue), 0 );
+        // By now libsolv supports orphan handling just in dup.
+        // We keep it here in _distupgrade to make sure nothing bad happens
+        // in case libsolv changes and it's used in remove commands which
+        // have no repos enabled. I.e. everything would be orphaned.
+        if (_removeOrphaned) {
+          queue_push( &(_jobQueue), SOLVER_DROP_ORPHANED|SOLVER_SOLVABLE_ALL);
+          queue_push( &(_jobQueue), 0 );
+        }
     }
-    if (_distupgrade_removeunsupported) {
-        queue_push( &(_jobQueue), SOLVER_DROP_ORPHANED|SOLVER_SOLVABLE_ALL);
-        queue_push( &(_jobQueue), 0 );
+    if (_removeUnneeded) {
+        invokeOnEach ( _pool.begin(), _pool.end(), [this]( const PoolItem & pi_r ) {
+          if ( pi_r.status().isUnneeded() ) {
+            queue_push( &(_jobQueue), SOLVER_ERASE | SOLVER_SOLVABLE_NAME | SOLVER_WEAK | MAYBE_CLEANDEPS );
+            queue_push( &(_jobQueue), pi_r.ident().id() );
+          }
+          return true;
+        } );
     }
 
     solverSetFocus( *_satSolver, _focus );
@@ -707,7 +723,7 @@ SATResolver::solving(const CapabilitySet & requires_caps,
       // Produtcs unless removeunsupported is active (cleans up all).
       if ( _distupgrade )
       {
-        if ( _distupgrade_removeunsupported )
+        if ( _removeOrphaned )
           MIL << "Droplist processing not needed. RemoveUnsupported is On." << endl;
         else if ( ! ZConfig::instance().solverUpgradeRemoveDroppedPackages() )
           MIL << "Droplist processing is disabled in ZConfig." << endl;
@@ -1046,7 +1062,7 @@ struct FindPackage
         {
         }
 
-    bool operator()( PoolItem p)
+    bool operator()( const PoolItem& p)
    {
        problemSolution->addSingleAction (p, action);
        return true;
@@ -1126,7 +1142,7 @@ std::string SATResolver::SATproblemRuleInfoString (Id probr, std::string &detail
 {
   std::string ret;
   sat::detail::CPool *pool = _satSolver->pool;
-  Id dep, source, target;
+  Id dep = 0, source = 0, target = 0;
   SolverRuleinfo type = solver_ruleinfo(_satSolver, probr, &source, &target, &dep);
 
   ignoreId = 0;
@@ -1342,9 +1358,9 @@ SATResolver::problems ()
     ResolverProblemList resolverProblems;
     if (_satSolver && solver_problem_count(_satSolver)) {
         sat::detail::CPool *pool = _satSolver->pool;
-        int pcnt;
-        Id p, rp, what;
-        Id problem, solution, element;
+        int pcnt = 0;
+        Id p = 0, rp = 0, what = 0;
+        Id problem = 0, solution = 0, element = 0;
         sat::Solvable s, sd;
 
         CapabilitySet system_requires = SystemCheck::instance().requiredSystemCap();
@@ -1357,7 +1373,7 @@ SATResolver::problems ()
             MIL << "Problem " <<  pcnt++ << ":" << endl;
             MIL << "====================================" << endl;
             std::string detail;
-            Id ignoreId;
+            Id ignoreId = 0;
             std::string whatString = SATprobleminfoString (problem,detail,ignoreId);
             MIL << whatString << endl;
             MIL << "------------------------------------" << endl;
